﻿namespace TabsStudioTracer
{
    public class Tracer : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IConfigurable
    {
        public Tracer()
        {
            settings = new Settings();
            settings.traceTabEvents = true;
            settings.traceTabExtensionEvents = true;
            settings.traceTabsEvents = true;
        }

        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            Trace("OnConnection");
            engine = engine_;

            engine.TabExtensionCreated += OnTabExtensionCreated;
            engine.TabExtensionDestroyed += OnTabExtensionDestroyed;
            engine.TabExtensionRenamed += OnTabExtensionRenamed;

            engine.TabCreated += OnTabCreated;
            engine.TabDestroyed += OnTabDestroyed;
            engine.TabRenamed += OnTabRenamed;

            engine.TabsCreated += OnTabsCreated;
            engine.TabsDestroyed += OnTabsDestroyed;
        }

        public void OnDisconnection()
        {
            Trace("OnDisconnection");

            engine.TabExtensionCreated -= OnTabExtensionCreated;
            engine.TabExtensionDestroyed -= OnTabExtensionDestroyed;
            engine.TabExtensionRenamed -= OnTabExtensionRenamed;

            engine.TabCreated -= OnTabCreated;
            engine.TabDestroyed -= OnTabDestroyed;
            engine.TabRenamed -= OnTabRenamed;

            engine.TabsCreated -= OnTabsCreated;
            engine.TabsDestroyed -= OnTabsDestroyed;
        }

        public void Configure(System.Windows.Window parent)
        {
            Trace("Configure");
            SettingsDialog dialog = new SettingsDialog(settings);
            dialog.Owner = parent;
            dialog.ShowDialog();
        }

        private void OnTabExtensionCreated(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            if (settings.traceTabExtensionEvents)
                Trace("OnTabExtensionCreated " + GetTabExtensionName(e.TabExtension.Label));
        }

        private void OnTabExtensionDestroyed(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            if (settings.traceTabExtensionEvents)
                Trace("OnTabExtensionDestroyed " + GetTabExtensionName(e.TabExtension.Label));
        }

        private void OnTabExtensionRenamed(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            if (settings.traceTabExtensionEvents)
                Trace("OnTabExtensionRenamed " + GetTabExtensionName(e.TabExtension.Label));
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            if (settings.traceTabEvents)
                Trace("OnTabCreated " + GetTabName(e.Tab.TabItem));
        }

        private void OnTabDestroyed(object sender, TabsStudioExt.TabEventArgs e)
        {
            if (settings.traceTabEvents)
                Trace("OnTabDestroyed " + GetTabName(e.Tab.TabItem));
        }

        private void OnTabRenamed(object sender, TabsStudioExt.TabEventArgs e)
        {
            if (settings.traceTabEvents)
                Trace("OnTabRenamed " + GetTabName(e.Tab.TabItem));
        }

        private void OnTabsCreated(object sender, TabsStudioExt.TabsEventArgs e)
        {
            if (settings.traceTabsEvents)
                Trace("OnTabsCreated");
        }

        private void OnTabsDestroyed(object sender, TabsStudioExt.TabsEventArgs e)
        {
            if (settings.traceTabsEvents)
                Trace("OnTabsDestroyed");
        }

        private void Trace(string s)
        {
            System.Diagnostics.Trace.WriteLine("TabsStudioTracer " + s);
        }

        private string GetTabName(System.Windows.Controls.TabItem tabItem)
        {
        	System.Windows.Controls.DockPanel tabInternals = tabItem.Header as System.Windows.Controls.DockPanel;
            System.Windows.Controls.StackPanel tabNameGroup = tabInternals.Children[0] as System.Windows.Controls.StackPanel;
            System.Windows.Controls.Label tabName = tabNameGroup.Children[0] as System.Windows.Controls.Label;
            return tabName.Content as string;        	
        }

        private string GetTabExtensionName(System.Windows.Controls.Label label)
        {
            return label.Content as string;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private Settings settings;
    }
}
